/*****************************************************************************
 *   acelerometro.c:  Driver gestor del acelerómetro MMA7361
 *   Calculos basados en http://www.starlino.com/imu_guide.html
 *   Copyright(C) 2014, Jesús Nicolás
 *   All rights reserved.
 *
 *
******************************************************************************/

#include "lpc17xx.h"
#include "FreeRTOS.h"
#include "semphr.h"
#include "task.h"
#include <math.h>
#include "type.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "printf.h"
#include "Config.h"
#include "uart.h"
#include "timer.h"
#include "adc.h"
#include "acelerometro.h"


// La aceleración en el eje Z cuando el acelerometro esta perpendicular al eje de la tierra es de 1G (800 mv) que corresponde con la gravedad
//#define Voltage_Z_1G 800
#define Voltage_Z_1G 0.8
#define Vref 3.3
#define ADC12bit 4095
#define VzeroG 1.65

// Resolución (la variable resolución indicará con cual trabajamos 0-baja, 1-alta)
// #define Resolucion_Min	0.860
// #define Resolucion_Max	0.2215
#define Resolucion_Min	800
#define Resolucion_Max	206

int Resolucion = 0;
float x = 0, y = 0, z = 0;
float offset_X = 0, offset_Y = 0,offset_Z = 0;



/**************************************************************************//**
 ** Funcion  Init_Acelerometro Inicializa el dispositivo Acelerometro MMA7361
 ** param Res (1= Rango 206 mg/V   0=Rango 800mg/V)On_Debug (0,1)
 ** return none
 *********************************************************************/
uint32_t Inicia_Acelerometro(int Res){
	LPC_PINCON->PINSEL1 &= ~0x00300C30;
//  Pull-up desactivado (10)
	LPC_PINCON->PINMODE1 |=(1<< 5);
	LPC_PINCON->PINMODE1 |=(1<< 21);
	LPC_PINCON->PINMODE1 |=(1<< 11);

	Port_Ctr_ACL-> SetPuerto |=GS;  		 //  GS - Get Select (Pin 11 -  P018)

	Resolucion = Res;
	if (Res==0)
		GS_Off;
	else
		GS_On;

	Calibracion(1000);

    return(TRUE);
}


/*****************************************************************************
** Function name:		 Calibracion
**
** Descriptions:		calcula el offset en estando el acelerometro en horizonta
**
** parameters:			numero de lecturas a efectuar sobre las que calcular
** Returned value:		None
**
*****************************************************************************/

void Calibracion(int Lecturas) {
	ADC_reg_channels Lec_XYZ;


  int i;
  float CorX=0, CorY=0, CorZ=0;

  for(i=0; i<Lecturas;i++) {
	  ADCReadChannels(3, &Lec_XYZ);
	  CorX = CorX + Lec_XYZ.channel0;
	  CorY = CorY + Lec_XYZ.channel1;
	  CorZ = CorZ + Lec_XYZ.channel2;
  }
  offset_Z=CorZ/Lecturas;
  offset_Z = offset_Z - Voltage_Z_1G;
  offset_X=CorX/Lecturas;
  offset_Y=CorY/Lecturas;


  return;
}

/**************************************************************************//**
 ** Funcion  Lee los ejes acelerometro devolviendo la salida
 ** 			en la medida del ADC, medida calculad  y los grados de cada eje
 ** param   Num_Lecturas  Muestras a recoger
 ** param  ACL_Data registro con los datos
 ** return none
 ** Calculos basados en http://www.starlino.com/imu_guide.html
 * *************************************************************************/
void Read_Acelerometro (int  Num_Lecturas, A_Reg *Datos ){
	ADC_reg_channels Lec_XYZ;
	float CorX=0, CorY=0, CorZ=0;
	float CalX=0,CalY=0,CalZ=0;
	float R=0;
	int i=0;
	if ( Num_Lecturas==0)  Num_Lecturas=1;
	for(i=0; i< Num_Lecturas;i++) {
		ADCReadChannels(3, &Lec_XYZ);
	    CorX = CorX + (Lec_XYZ.channel0);
	    CorY = CorY + (Lec_XYZ.channel1);
	    CorZ = CorZ + (Lec_XYZ.channel2);
	}
	x=CorX/ Num_Lecturas;
	y=CorY/ Num_Lecturas;
	z=CorZ/ Num_Lecturas;

	Datos->Lectura_X=x;
	Datos->Lectura_Y=y;
	Datos->Lectura_Z=z;

	if (Resolucion==0) {
    	CalX = ((x - offset_X) * Vref / ADC12bit - VzeroG) / Resolucion_Min;
    	CalY = ((y - offset_Y) * Vref / ADC12bit - VzeroG) / Resolucion_Min;
    	CalZ = ((z - offset_Z) * Vref / ADC12bit - VzeroG) / Resolucion_Min;
//    	CalX = ((x) * Vref / ADC12bit - VzeroG) / Resolucion_Min;
//    	CalY = ((y) * Vref / ADC12bit - VzeroG) / Resolucion_Min;
//    	CalZ = ((z) * Vref / ADC12bit - VzeroG) / Resolucion_Min;

    } else
    {
    	CalX = ((x - offset_X) * Vref / ADC12bit - VzeroG) / Resolucion_Max;
    	CalY = ((y - offset_Y) * Vref / ADC12bit - VzeroG) / Resolucion_Max;
    	CalZ = ((z - offset_Z) * Vref / ADC12bit - VzeroG) / Resolucion_Max;
    }

	Datos->Procesado_X = CalX;
	Datos->Procesado_Y = CalY;
	Datos->Procesado_Z = CalZ;

	R = sqrt( (CalX*CalX) + (CalY*CalY) + (CalZ*CalZ) );
	Datos->Angulo_X = acos(CalX/R);
	Datos->Angulo_Y = acos(CalY/R);
	Datos->Angulo_Z = acos(CalZ/R);

//	Delay_Ms(Task, 10000);

    return;
}



