/*****************************************************************************
 *   Wifly.c
 *
 *   Copyright(C) 2014, Jesús Nicolás
 *   All rights reserved.
 *
 *
******************************************************************************/
#include "FreeRTOS.h"
#include "task.h"
#include "semphr.h"
#include "type.h"
#include "printf.h"
#include "wifly.h"
#include "config.h"
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "LPC17xx.h"

static char Buffer[GetSIZE];
static uint32_t Estat = NORMAL, Error_Send = 0, Error_Read = 0;

/* Declare a variable of type xSemaphoreHandle */
volatile static xSemaphoreHandle wMutex = NULL;

/*****************************************************************************
 ** Function name:		Lock_Wifly
 **
 ** Descriptions:		Use Semaphore or vTaskSuspend for block process
 **
 ** parameters:			None
 ** Returned value:		None
 **
 *****************************************************************************/
void Lock_Wifly() {
	if (wMutex == NULL ) {
		wMutex = xSemaphoreCreateRecursiveMutex();
	}
	xSemaphoreTakeRecursive(wMutex, portMAX_DELAY);
}

/*****************************************************************************
 ** Function name:		UnLock_Wifly
 **
 ** Descriptions:		Use Semaphore or vTaskResume for unblock process
 **
 ** parameters:			None
 ** Returned value:		None
 **
 *****************************************************************************/
void UnLock_Wifly() {
	xSemaphoreGiveRecursive(wMutex);
}

/*****************************************************************************
 ** Function name:		Wifly_Reset
 **
 ** Descriptions:		Hardware reset the WiFly device. PIN 6-24 (P0.22) of LPC1769
 **
 ** parameters:			None
 ** Returned value:		None
 **
 *****************************************************************************/
void Wifly_Reset() {
	// Initialize pin
	LPC_PINCON ->PINSEL1 &= (~(3 << 12));
	LPC_GPIO0 ->FIODIR |= (1 << WIFLY_PIN_RST);

	// it made reset at Wifly
	LPC_GPIO0 ->FIOCLR = 1 << WIFLY_PIN_RST;
	Delay_Ms(Brute, 1); // min 160us
	LPC_GPIO0 ->FIOSET = 1 << WIFLY_PIN_RST;
}

/*****************************************************************************
 ** Function name:		Wifly_Init
 **
 ** Descriptions:		Defines the used UART, its baudrate and reset device
 ** 						This operation produce a few delay.
 **
 ** parameters:			UART port ,  desired baudrate
 ** Returned value:		TRUE or FALSE success
 **
 *****************************************************************************/
uint32_t Wifly_Init(uint8_t PortNum, uint32_t Baudrate) {
	uint32_t ret = FALSE;

	/* Prepared to create a mutex type semaphor to avoid concurrent access to wifly */
	Lock_Wifly();
	{
		Print_Flush(PortNum);
		ret = Print_Init(PortNum, Baudrate);
		Wifly_Reset();
		Delay_Ms(Brute, 500);  // min 160us
		Print_Flush(PortNum);
	}
	UnLock_Wifly();
	return ret;
}

/*****************************************************************************
 ** Function name:		Wifly_End
 **
 ** Descriptions:		Close the UART port used for WiFly device.
 **
 ** parameters:			UART port
 ** Returned value:		TRUE or FALSE success
 **
 *****************************************************************************/
uint32_t Wifly_End(uint8_t PortNum) {
	uint32_t ret = FALSE;

	/* Prepared to create a mutex type semaphor to avoid concurrent access to wifly */
	Lock_Wifly();
	{
		Print_Flush(PortNum);
		ret = Print_End(PortNum);
		Print_Flush(PortNum);
	}
	UnLock_Wifly();
	return ret;
}

/*****************************************************************************
 ** Function name:		Wifly_Set
 **
 ** Descriptions:		Send Cmd (pointer char)to UART, wait a time for receive response
 ** 						and check response with pointer Value
 **
 ** parameters:			UART port to send, char pointer to command and time to wait response
 ** Returned value:		TRUE or FALSE success
 **
 *****************************************************************************/
uint32_t Wifly_Set(uint8_t PortNum, char *Comand, uint32_t Time) {
	uint32_t ret = FALSE;
	Lock_Wifly();
	Print_Flush(PortNum);
	//Buffer[0] = '\0';
	Print_T_CRLF(PortNum, Comand);
	if (UART_Read(PortNum, (char *) Buffer, (char *) "\n", Time) > 0)
		ret = TRUE;
	UnLock_Wifly();
	return ret;
}

/*****************************************************************************
 ** Function name:		Wifly_Set_OK
 **
 ** Descriptions:		Send Cmd (pointer char)to UART, wait a time for receive response
 ** 						and check response with pointer Value
 **
 ** parameters:			UART port to send, char pointer to command, and time to wait response
 ** Returned value:		TRUE or FALSE success
 **
 *****************************************************************************/
uint32_t Wifly_Set_OK(uint8_t PortNum, char *Comand, uint32_t Time) {
	uint32_t ret = FALSE;
	Lock_Wifly();
	Print_Flush(PortNum);
	//Buffer[0] = '\0';
	Print_T_CRLF(PortNum, Comand);
	ret = Check_Read(PortNum, (char *) Buffer, (char *) "AOK", Time);
	UnLock_Wifly();
	return ret;
}

/*****************************************************************************
 ** Function name:		Wifly_Set_TT_OK
 **
 ** Descriptions:		Send Cmd (pointer char)to UART, wait a time for receive response
 ** 						and check response with pointer Value
 **
 ** parameters:			UART port to send, char pointer to command, char pointer to join at command
 ** 						and time to wait response
 ** Returned value:		TRUE or FALSE success
 **
 *****************************************************************************/
uint32_t Wifly_Set_TT_OK(uint8_t PortNum, char *Comand, char *Value,
		uint32_t Time) {
	uint32_t ret = FALSE;
	Lock_Wifly();
	Print_Flush(PortNum);
	Print_TT_CRLF(PortNum, Comand, Value);
	ret = Check_Read(PortNum, (char *) Buffer, (char *) "AOK", Time);
	UnLock_Wifly();
	return ret;
}

/*****************************************************************************
 ** Function name:		Wifly_Set_TT_OK
 **
 ** Descriptions:		Send Cmd (pointer char)to UART, wait a time for receive response
 ** 						and check response with pointer Value
 **
 ** parameters:			UART port to send, char pointer to command, number to join at command
 ** 						and time to wait response
 ** Returned value:		TRUE or FALSE success
 **
 *****************************************************************************/
uint32_t Wifly_Set_TN_OK(uint8_t PortNum, char *Comand, int32_t Num,
		uint32_t Time) {
	uint32_t ret = FALSE;
	Lock_Wifly();
	Print_Flush(PortNum);
	//Buffer[0] = '\0';
	Print_TN_CRLF(PortNum, Comand, Num);
	ret = Check_Read(PortNum, (char *) Buffer, (char *) "AOK", Time);
	UnLock_Wifly();
	return ret;
}

/*****************************************************************************
 ** Function name:		Wifly_Set_Check
 **
 ** Descriptions:		Send Cmd (pointer char)to UART, wait a time for receive response
 ** 						and check response with pointer Value
 **
 ** parameters:			UART port to send, char pointer to command, char pointer to check
 ** 						and time to wait response
 ** Returned value:		TRUE or FALSE success
 **
 *****************************************************************************/
uint32_t Wifly_Set_Check(uint8_t PortNum, char *Comand, char *Check,
		uint32_t Time) {
	uint32_t ret = FALSE;
	Lock_Wifly();
	Print_Flush(PortNum);
	//Buffer[0] = '\0';
	Print_T_CRLF(PortNum, Comand);
	ret = Check_Read(PortNum, (char *) Buffer, Check, Time);
	UnLock_Wifly();
	return ret;
}

/*****************************************************************************
 ** Function name:		Wifly_Set_TT_Check
 **
 ** Descriptions:		Send Cmd (pointer char)to UART, wait a time for receive response
 ** 						and check response with pointer Value
 **
 ** parameters:			UART port to send, char pointer to command, char pointer Value to join at command
 ** 						char pointer to Check, and time to wait response
 ** Returned value:		TRUE or FALSE success
 **
 *****************************************************************************/
uint32_t Wifly_Set_TT_Check(uint8_t PortNum, char *Comand, char *Value,
		char *Check, uint32_t Time) {
	uint32_t ret = FALSE;
	Lock_Wifly();
	Print_Flush(PortNum);
	//Buffer[0] = '\0';
	Print_TT_CRLF(PortNum, Comand, Value);
	ret = Check_Read(PortNum, (char *) Buffer, Check, Time);
	UnLock_Wifly();
	return ret;
}

/*****************************************************************************
 ** Function name:		Wifly_CMD
 **
 ** Descriptions:		Send '$$$' to enter at Command Mode of WiFly
 **
 ** parameters:			UART port to send
 ** Returned value:		TRUE or FALSE success
 **
 *****************************************************************************/
uint32_t Wifly_CMD(uint8_t PortNum) {
	uint32_t ret = FALSE;
	Lock_Wifly();
	if (Estat == COMMAND)
		return TRUE;
	Print_Flush(PortNum);
	//Buffer[0] = '\0';
	Print_Text(PortNum, (char *) "$$$");
	ret = Check_Read(PortNum, (char *) Buffer, (char *) "CMD", 300);
	if (ret == TRUE)
		Estat = COMMAND;
	UnLock_Wifly();
	return ret;
}

/*****************************************************************************
 ** Function name:		Wifly_Exit
 **
 ** Descriptions:		Send 'exit' to return at Serial Mode of WiFly
 **
 ** parameters:			UART port to send
 ** Returned value:		TRUE or FALSE success
 **
 *****************************************************************************/
uint32_t Wifly_Exit(uint8_t PortNum) {
	uint32_t ret = FALSE;
	Lock_Wifly();
	if (Estat == NORMAL)
		ret = TRUE;
	else
	{
		Print_Flush(PortNum);
		Print_T_CRLF(PortNum, (char *)"exit");
		ret = Check_Read(PortNum, (char *)Buffer,(char *)"EXIT", 1000);
		if (ret == TRUE)
			Estat = NORMAL;
			else Estat = COMMAND;  //JNJ
	}
	UnLock_Wifly();
	return ret;
}

/*****************************************************************************
 ** Function name:		Wifly_Open
 **
 ** Descriptions:		Send 'open' with Address (empty or text)
 **
 ** parameters:			UART port to send and Address to connect
 ** Returned value:		TRUE or FALSE success
 **
 *****************************************************************************/
uint32_t Wifly_Open(uint8_t PortNum, char *Address) {
	uint32_t ret = FALSE;
	Lock_Wifly();
//	Print_T_CRLF(UART3, "OPEN 1" );
	if (Estat == NORMAL) Wifly_CMD(PortNum);

//	Print_T_CRLF(UART3, "OPEN 2" );
	Wifly_Set_OK(PortNum, (char *)"set c r 0", 1000);
//	Print_T_CRLF(UART3, "OPEN 3" );
	Print_Flush(PortNum);
//	Print_T_CRLF(UART3, "OPEN 4" );
	//Buffer[0] = '\0';
	if (strlen(Address) > 0)
	{
//		Print_T_CRLF(UART3, "OPEN 4.1" );
		Print_T_CRLF(UART_CON, Address );
//		Print_T_CRLF(UART3, "OPEN 4.1.1" );
		Print_TT_CRLF(PortNum, (char *)"open", Address);
//		Print_T_CRLF(UART3, "OPEN 4.2" );
	}
	else
	{
//		Print_T_CRLF(UART3, "OPEN 4.3" );
		Print_T_CRLF(PortNum, (char *)"open");
//		Print_T_CRLF(UART3, "OPEN 4.4" );
	}
//	Print_T_CRLF(UART3, "OPEN 5" );
	ret = Check_Read(PortNum, (char *)Buffer, (char *)"*OPEN*", 500);
//	Print_T_CRLF(UART3, "OPEN 6" );
	if (ret == TRUE) Estat = NORMAL;
//	Print_T_CRLF(UART3, "OPEN 7" );
	UnLock_Wifly();
	return ret;
}

/*****************************************************************************
 ** Function name:		Wifly_Close
 **
 ** Descriptions:		Send 'close' to close open TCP connection
 **
 ** parameters:			UART port to send
 ** Returned value:		TRUE or FALSE success
 **
 *****************************************************************************/
uint32_t Wifly_Close(uint8_t PortNum) {
	uint32_t ret = FALSE;
	Lock_Wifly();
	if (Estat == NORMAL) Wifly_CMD(PortNum);
	Print_Flush(PortNum);
	//Buffer[0] = '\0';
	Print_T_CRLF(PortNum, (char *)"close");
	ret = Check_Read(PortNum, (char *)Buffer, (char *)"*CLOS*", 300);
	UnLock_Wifly();
	return ret;
}

/*****************************************************************************
 ** Function name:		Wifly_Save
 **
 ** Descriptions:		Send 'save' to store actual configuration
 **
 ** parameters:			UART port to send
 ** Returned value:		TRUE or FALSE success
 **
 *****************************************************************************/
uint32_t Wifly_Save(uint8_t PortNum) {
	uint32_t ret = FALSE;

	Lock_Wifly();
	if (Estat == NORMAL)
		Wifly_CMD(PortNum);
	Print_Flush(PortNum);
	Print_T_CRLF(PortNum, (char *)"save");

	ret = Check_Read(PortNum, (char *)Buffer, (char *)"Storing in config", 1000);

	UnLock_Wifly();
	return ret;
}

/*****************************************************************************
 ** Function name:		Wifly_Reboot
 **
 ** Descriptions:		Send 'reboot' now initialize the WiFly
 **
 ** parameters:			UART port to send
 ** Returned value:		TRUE or FALSE success
 **
 *****************************************************************************/
uint32_t Wifly_Reboot(uint8_t PortNum) {
	uint32_t ret = FALSE;

	Lock_Wifly();

	if (Estat== NORMAL)
		Wifly_CMD(PortNum);

	ret = Wifly_Set_Check(PortNum, (char*)"reboot", (char*)"*READY*", 2000);

	if (ret == TRUE)
		Estat = NORMAL;

	UnLock_Wifly();
	return ret;
}

/*****************************************************************************
 ** Function name:		Wifly_Connect
 **
 ** Descriptions:		Connect to a Wi-Fy network
 **
 ** parameters:			Pointer to Buffer and Time to wait in milliseconds
 ** Returned value:		Number of characters read
 **
 *****************************************************************************/
uint32_t Wifly_Connect(uint8_t PortNum, uint32_t Mode, char *ssid, char* Phrase) {
	uint32_t ret = FALSE;
	Lock_Wifly();
	if (Estat == NORMAL)
		Wifly_CMD(PortNum);
	Wifly_Set_TN_OK(PortNum, (char *) "set w a", Mode, 500);
	Wifly_Set_TT_OK(PortNum, (char *) "set w s", ssid, 500);
	Wifly_Set_TT_OK(PortNum, (char *) "set w p", Phrase, 750);
	ret = Wifly_Set_TT_Check(PortNum, (char *) "join", ssid,
			(char *) "Associated!", 9000);
	ret = Check_Read(PortNum, (char *) Buffer, (char *) "AOK", 1000); //JNJ

	UnLock_Wifly();
	return ret;
}

/*****************************************************************************
 ** Function name:		Wifly_AutoConnect
 **
 ** Descriptions:		Connect to a Wi-Fy network mode Auto at Start
 **
 ** parameters:			Pointer to Buffer and Time to wait in milliseconds
 ** Returned value:		Number of characters read
 **
 *****************************************************************************/
uint32_t Wifly_AutoConnect(uint8_t PortNum, uint32_t Mode, char* Phrase) {
	uint32_t ret = FALSE;
	Lock_Wifly();
	if (Estat == NORMAL)
		Wifly_CMD(PortNum);
	Wifly_Set_TN_OK(PortNum, (char *) "set w a", Mode, 500);
	Wifly_Set_TT_OK(PortNum, (char *) "set w p", Phrase, 750);
	Wifly_Set_OK(PortNum, (char *) "set w j 1", 500);
	ret = Wifly_Set_Check(PortNum, (char *) "join", (char *) "Associated!",
			5000);
	if (ret == TRUE)
		ret = Wifly_Save(PortNum);
	UnLock_Wifly();
	return ret;
}

/*****************************************************************************
 ** Function name:		Wifly_Disconnect
 **
 ** Descriptions:		Send 'leave' to disconnect the WLAN
 **
 ** parameters:			UART port to send
 ** Returned value:		TRUE or FALSE success
 **
 *****************************************************************************/

uint32_t Wifly_Disconnect(uint8_t PortNum) {
	uint32_t ret = FALSE;
	Lock_Wifly();
	if (Estat == NORMAL)
		Wifly_CMD(PortNum);
	ret = Wifly_Set_Check(PortNum, (char*) "leave", (char *) "DeAuth", 500);
	UnLock_Wifly();
	return ret;
}

/*****************************************************************************
 ** Function name:		Wifly_Ping
 **
 ** Descriptions:		Send 'ping' with Address
 **
 ** parameters:			UART port to send and Address to check ping
 ** Returned value:		TRUE or FALSE success
 **
 *****************************************************************************/
uint32_t Wifly_Ping(uint8_t PortNum, char *Address) {
	uint32_t ret = FALSE;
	Lock_Wifly();
	if (Estat == NORMAL)
		Wifly_CMD(PortNum);
	ret = Wifly_Set_TT_Check(PortNum, (char *) "ping", Address, (char *) "PING",
			2000);
#ifdef MODE_DEBUG
	if (ret == TRUE) {
		char *string;
		string = strstr(Buffer, (char *) "PING");
		string = strtok(string, (char *) "\r\n");
		Print_TT_CRLF(UART_CON, (char *) "El Resultado de Ping es :", string);
	}
#endif
	UnLock_Wifly();
	return ret;
}

/*****************************************************************************
 ** Function name:		Wifly_IP
 **
 ** Descriptions:		Send 'get ip' to know the IP used in WLAN
 **
 ** parameters:			UART port to send
 ** Returned value:		TRUE or FALSE success
 **
 *****************************************************************************/
uint32_t Wifly_IP(uint8_t PortNum) {
	uint32_t ret = FALSE;
	//TODO
	return ret;
}

/*****************************************************************************
 ** Function name:		Wifly_Channel
 **
 ** Descriptions:		Send 'show net' to know the Channel used in WLAN
 **
 ** parameters:			UART port to send
 ** Returned value:		TRUE or FALSE success
 **
 *****************************************************************************/
uint32_t Wifly_Channel(uint8_t PortNum) {
	uint32_t ret = FALSE;
	//TODO
	return ret;
}

/*****************************************************************************
 ** Function name:		Wifly_RSSI
 **
 ** Descriptions:		Send 'show rssi' to know the signal RSSI in dBm
 **
 ** parameters:			UART port to send
 ** Returned value:		RSSI Value or 0 if not found
 **
 *****************************************************************************/
int32_t Wifly_RSSI(uint8_t PortNum) {
	int32_t ret = 0;
	//TODO
	return ret;
}

/*****************************************************************************
 ** Function name:		Wifly_UpTime
 **
 ** Descriptions:		Send 'show time' to know the signal RSSI in dBm
 **
 ** parameters:			UART port to send
 ** Returned value:		UpTime Value or 0 if not found
 **
 *****************************************************************************/
int32_t Wifly_UpTime(uint8_t PortNum) {
	uint32_t ret = 0;
	//TODO
	return ret;
}

/*****************************************************************************
 ** Function name:		Wifly_GetPower
 **
 ** Descriptions:		Send 'get wlan' to know the transmission Power
 **
 ** parameters:			UART port to send
 ** Returned value:		TRUE or FALSE success
 **
 *****************************************************************************/
uint32_t Wifly_GetPower(uint8_t PortNum) {
	uint32_t ret = FALSE;
	//TODO
	return ret;
}

/*****************************************************************************
 ** Function name:		Wifly_SetPower
 **
 ** Descriptions:		Send 'set wlan tx XX' to  the transmission Power
 ** 						where XX is a range 0-12. Default 0 = 12 dBm (Max Value)
 **
 ** parameters:			UART port to send
 ** Returned value:		TRUE or FALSE success
 **
 *****************************************************************************/
uint32_t Wifly_SetPower(uint8_t PortNum, uint32_t Value) {
	uint32_t ret = FALSE;
	//TODO
	return ret;
}

/*****************************************************************************
 ** Function name:		Wifly_BR
 **
 ** Descriptions:		 instantaneous Baudrate with 'set uart instant <value>'
 **
 ** parameters:			UART port to send , new desired Baudrate
 ** Returned value:		TRUE or FALSE success
 **
 *****************************************************************************/

uint32_t Wifly_ChangeBR(uint8_t PortNum, int32_t Value) {
	uint32_t ret = FALSE;
	Lock_Wifly();

	if (Estat == NORMAL)
		Wifly_CMD(PortNum);
	Print_Flush(PortNum);

	Wifly_Set_TN_OK(PortNum, (char *)"set u b", Value, 800);

	Wifly_Save(PortNum);
	Wifly_Reboot(PortNum);
	Wifly_End(PortNum);
	ret = Wifly_Init(PortNum, Value);
	Delay_Ms(DELAY,2000);
	//Delay_Ms(DELAY,500);
	Estat = NORMAL;

	UnLock_Wifly();

	return ret;
}

/*****************************************************************************
 ** Function name:		Wifly_GET_Telnet
 **
 ** Descriptions:		Connect to specified Host and parameters (CGI address) with Mode HTTP/1.0 or HTTP/1.1
 **
 ** parameters:			UART port to send, HOST to connect, sub-address in Host, Mode HTTP
 ** Returned value:		TRUE or FALSE success
 **
 *****************************************************************************/
uint32_t Wifly_GET_Telnet(uint8_t PortNum, char *Host, char *Param) {
	uint32_t ret = FALSE;
	//TODO
	return ret;
}

/*****************************************************************************
 ** Function name:		Wifly_GET_Base
 **
 ** Descriptions:		Connect to specified Host and parameters (CGI address) with Mode HTTP/1.0 or HTTP/1.1
 **
 ** parameters:			UART port to send, HOST to connect, sub-address in Host, Mode HTTP
 ** Returned value:		TRUE or FALSE success
 **
 *****************************************************************************/
uint32_t Wifly_GET_Base(uint8_t PortNum, char *Host, char *Param) {
	uint32_t ret = FALSE;
	//TODO
	return ret;
}

/*****************************************************************************
 ** Function name:		Wifly_Web_Send
 **
 ** Descriptions:		Connect to specified Host and parameters (CGI address), and add Value to send
 ** 						with Mode HTTP/1.1
 **
 ** parameters:			UART port to send, HOST to connect, sub-address in Host, Value to add
 ** Returned value:		TRUE or FALSE success
 **
 *****************************************************************************/
uint32_t Wifly_Web_Send(uint8_t PortNum, char *Host, char *Param, uint32_t Value) {
//JNJ
	uint32_t ret = FALSE;
	char buf[SendSIZE];
	Lock_Wifly();
	if (Estat == NORMAL) Wifly_CMD(PortNum);
	Wifly_CMD(PortNum);
	sprintf(buf, "%s %s", Host, (char *)NetPORT);
	Wifly_Open(PortNum, buf);
	Print_Flush(PortNum);
	sprintf(buf, "%s %s%u %s\r\n%s %s\r\n\r\n", (char *)"GET", Param, Value, (char *)WEB, (char *)"host:", Host);
	Print_T_CRLF(PortNum, buf);
	ret = Check_Read(PortNum, (char *)Buffer,(char *)"200 OK", 4000);
#ifdef MODE_DEBUG
	if (ret == TRUE)
	{
		char *string;
		string = strstr(Buffer, (char *)"HTTP");
		string = strtok(string, (char *)"\r\n");
		Print_TT_CRLF(UART_CON, (char *)"EL resultado del Send es :", string);
	}
#endif
	Wifly_Close(PortNum);
	UnLock_Wifly();
	return ret;
}

/*****************************************************************************
 ** Function name:		Wifly_Web_Read
 **
 ** Descriptions:		Connect to specified Host and parameters (CGI address) with Mode HTTP/1.1
 ** 						and collect received Value.
 **
 ** parameters:			UART port to send, HOST to connect, sub-address in Host
 ** Returned value:		Value received or 0 if not correct
 **
 *****************************************************************************/
int32_t Wifly_Web_Read(uint8_t PortNum, char *Host, char *Param) {
//JNJ
	uint32_t ret = 0;
	char buf[GetSIZE];
//	char buf[SendSIZE];

	char *parse;
	Lock_Wifly();

	if (Estat != COMMAND) Wifly_CMD(PortNum);
//	Wifly_CMD(PortNum);
	sprintf(buf, "%s %s", Host, (char *)NetPORT);
	Wifly_Open(PortNum, buf);
	Print_Flush(PortNum);
	sprintf(buf, "%s %s %s\r\n%s %s\r\n\r\n", (char *)"GET", Param, (char *)WEB, (char *)"host:", Host);
	Print_T_CRLF(PortNum, buf);

	if (Check_Read(PortNum, (char *)Buffer,(char *)"200 OK", 1000) == TRUE)
	{
		strcpy(buf, Buffer);
	#ifdef MODE_DEBUG
		char *string;
		string = strstr(Buffer, (char *)"HTTP");
		string = strtok(string, (char *)"\r\n");
		Print_TT_CRLF(UART_CON, (char *)"El resultado del Read es :", string);
	#endif
		parse = strstr(buf, (char *)"chunked");
		if  (parse == NULL)
		{
			parse = strstr(buf, (char *)"private");
			parse = strtok(parse, (char *)"*CLOS*");
			parse = strtok(parse, (char *)"\r\n");
			parse = strtok(NULL, (char *)"\r\n");
		}
		else
		{
			parse = strtok(parse, (char *)"\r\n");
			parse = strtok(NULL, (char *)"\r\n");
			parse = strtok(NULL, (char *)"\r\n");
		}
		ret = atoi(parse);
	}
	Wifly_Close(PortNum);
	UnLock_Wifly();
	return ret;
}

/*****************************************************************************
 ** Function name:		Wifly_Udp_Send
 **
 ** Descriptions:		Send UPD packet to remote nodes with new led value
 **
 ** parameters:			UART port to send, key, device, led
 ** Returned value:		TRUE or FALSE success
 **
 *****************************************************************************/
uint32_t Wifly_Udp_Send(uint8_t PortNum, char *key, char *dispositivo, uint8_t led) {
//JNJ
	uint32_t ret = FALSE;
	char buf[SendSIZE];

	Lock_Wifly();

	if (Estat == NORMAL) Wifly_CMD(PortNum);

// El valor del led está delimitado entre "LED" y "<"
//	sprintf(buf, "%s%s%s%d%s\r\n",(char *)"set o d JNJ", (char *)dispositivo,(char *)"LED", (uint8_t )led, "<");
	sprintf(buf, "%s%s%s%s%d%s\r\n",(char *)"set o d ", (char *)key, (char *)dispositivo,(char *)"LED", (uint8_t )led, "<");
	ret = Wifly_Set_OK(PortNum, buf, 500);


#ifdef MODE_DEBUG
	if (ret == TRUE)
	{
		char *string;
		string = strstr(Buffer, (char *)"AOK");
		string = strtok(string, (char *)"\r\n");
		Print_TT_CRLF(UART_CON, (char *)" (MW) El resultado del Send UDP es:", string);
	}
#endif

	if (ret != TRUE){ //Hay problemas
		Wifly_Reboot(UART_WIFLY);
		Wifly_CMD(PortNum);
		Wifly_Set_OK(PortNum, (char *)"set ip protocol 1", 1000);
		ret = Wifly_Set_OK(PortNum, buf, 500);
		char *string;
		string = strstr(Buffer, (char *)"AOK");
		string = strtok(string, (char *)"\r\n");
#ifdef MODE_DEBUG
		Print_TN_CRLF(UART_CON, (char *)" (MW) EL resultado del reintento del Send UDP es:", ret);
#endif
	}
	if (ret == TRUE)
	{
		ret = Wifly_SW_Broadcast(PortNum, 1);
	}


#ifdef MODE_DEBUG
	if (ret == TRUE)
	{
		char *string;
		string = strstr(Buffer, (char *)"AOK");
		string = strtok(string, (char *)"\r\n");
		Print_TT_CRLF(UART_CON, (char *)" (MW) El resultado final del Send UDP es :", string);
	}
#endif

	UnLock_Wifly();

	return ret;
}

/*****************************************************************************
 ** Function name:		Wifly_Udp_Read
 **
 ** Descriptions:		Read UPD packet from remote nodes with new led value
 **
 **
 ** parameters:			UART port to send, key private lan
 ** Returned value:		remote led received
 **
 *****************************************************************************/
int32_t Wifly_Udp_Read(uint8_t PortNum, char *Key) {

	uint32_t ret = 0;
	char buf[BroadcastSIZE];
//	broadcast_Reg broad;
	char *parse;

	Lock_Wifly();

	Wifly_Exit(PortNum);
	ret = Check_Read(PortNum, (char *)buf,(char *)Key, 1000);
	if (ret == TRUE)
	{
		char *string;
		string = strstr(buf, (char *)"LED");
		string = strtok(string, (char *)"<");
#ifdef MODE_DEBUG
		Print_TT_CRLF(UART_CON, (char *)" (MW) EL resultado de READ es :", string);
#endif
		parse = strtok(string, (char *)"LED");
		ret = atoi(parse);
	} else {
		ret = 0;
	}

	UnLock_Wifly();
	return ret;

}



/*****************************************************************************
 ** Function name:		Wifly_SW_Broadcast
 **
 ** Descriptions:		Set broadcast interval
 **
 ** parameters:			UART port to send, interval in seconds
 ** Returned value:		TRUE or FALSE success
 **
 *****************************************************************************/
uint32_t Wifly_SW_Broadcast(uint8_t PortNum, uint8_t Param) {
	uint32_t ret = FALSE;
	char buf[SendSIZE];

	Lock_Wifly();

	if (Estat == NORMAL) Wifly_CMD(PortNum);

	sprintf(buf, "%s%d\r\n",(char *)"set broadcast interval ",  (uint8_t )Param);
	ret = Wifly_Set_OK(PortNum, buf, 500);

	UnLock_Wifly();
	return ret;
}
/*****************************************************************************
 ** Function name:		Wifly_Auto_Rol
 **
 ** Descriptions:		Set the Rol (APNODE or CLIENT) automatically
 **
 ** parameters:			UART port to send, SSID
 ** Returned value:		TRUE
 **
 *****************************************************************************/

uint32_t Wifly_Auto_Rol(uint8_t PortNum, char *Ssid) {
	uint32_t ret = FALSE;
	char buf[TipSIZE];
	char command[SendSIZE];

	Lock_Wifly();
	Print_Flush(PortNum);
	if (Estat == NORMAL) Wifly_CMD(PortNum);
	Print_T_CRLF(PortNum, (char *)"scan");
	ret = Check_Read(PortNum, (char *)buf,(char *)Ssid, 4000);
	Print_TT_CRLF(UART_CON, (char *)" (MW) EL resultado de SCSN es 1----> :", buf);
	if (ret == TRUE)
		{

	     Print_T_CRLF(UART_CON, (char *)"------RESET CLIENT------------------------------------");
	     Wifly_Reset();
		 Wifly_Set_OK(PortNum, (char *)"set ip dhcp 1", 300);
	   	 sprintf(command, "%s%s\r\n",(char *)"set wlan ssid  ",  (char *)Ssid);
	 	 ret = Wifly_Set_OK(PortNum, command, 300);

	     Wifly_Set_OK(PortNum, (char *)"set option deviceid JNJ02LED0< ", 300);
	     Wifly_Set_OK(PortNum, (char *)"set broadcast interval 10 ", 300);
		 Wifly_Set_OK(PortNum, (char *)"set broadcast port 55555 ", 300);
		 Wifly_Set_OK(PortNum, (char *)"set ip remote 55555 ", 300);
		 Wifly_Set_OK(PortNum, (char *)"set ip local 55555 ", 300);
		 Wifly_Set_OK(PortNum, (char *)"set wlan phrase matadorjj", 400);
	     Wifly_Set_OK(PortNum, (char *)"set ip proto 1 ", 300);
	     Wifly_Set_OK(PortNum, (char *)"set wlan join 1 ", 300);
	     Wifly_Save(PortNum);
	     Wifly_Reboot(PortNum);


		} else {
	   		ret = TRUE;
		    Print_T_CRLF(UART_CON, (char *)"------RESET APMODE------------------------------------");
		    Wifly_Reset();
			Wifly_Set_OK(PortNum, (char *)"set wlan join 7", 300);
			Wifly_Set_OK(PortNum, (char *)"set wlan channel 3", 300);
		   	sprintf(command, "%s%s\r\n",(char *)"set apmode ssid  ",  (char *)Ssid);
		 	ret = Wifly_Set_OK(PortNum, command, 300);
			Wifly_Set_OK(PortNum, (char *)"set apmode passphrase matadorjj", 300);
			Wifly_Set_OK(PortNum, (char *)"set ip dhcp 4", 300);
			Wifly_Set_OK(PortNum, (char *)"set ip address 192.168.20.1", 300);
			Wifly_Set_OK(PortNum, (char *)"set ip net 255.255.255.0", 300);
			Wifly_Set_OK(PortNum, (char *)"set ip gateway 192.168.20.1", 300);
			Wifly_Set_OK(PortNum, (char *)"set option deviceid JNJ01LED0<", 300);
			Wifly_Set_OK(PortNum, (char *)"set broadcast interval 10", 300);
			Wifly_Set_OK(PortNum, (char *)"set broadcast port 55555", 300);
			Wifly_Set_OK(PortNum, (char *)"set ip remote 55555", 300);
			Wifly_Set_OK(PortNum, (char *)"set ip local 55555", 300);
			Wifly_Set_OK(PortNum, (char *)"set ip proto 1", 300);
		    Wifly_Save(PortNum);
		    Wifly_Reboot(PortNum);
		}

	UnLock_Wifly();
	return ret;

}
/*****************************************************************************
 ** Function name:		Wifly_Send_Error
 **
 ** Descriptions:		Check the number of errors in send function and Initialize value.
 **
 ** parameters:			Limit to check and True o false for initialize
 ** Returned value:		TRUE or FALSE success
 **
 *****************************************************************************/
uint32_t Wifly_Send_Error(uint32_t Limit, uint32_t Init) {
	uint32_t ret = FALSE;
	Lock_Wifly();
	{
		if (Error_Send > Limit)
			ret = TRUE;
		if (Init == TRUE)
			Error_Send = 0;
	}
	UnLock_Wifly();
	return ret;
}



/*****************************************************************************
 ** Function name:		Wifly_Read_Error
 **
 ** Descriptions:		Check the number of errors in read function and Initialize value.
 **
 ** parameters:			Limit to check and True o False for initialize
 ** Returned value:		TRUE or FALSE success
 **
 *****************************************************************************/
uint32_t Wifly_Read_Error(uint32_t Limit, uint32_t Init) {
	uint32_t ret = FALSE;
	Lock_Wifly();
	{
		if (Error_Read > Limit)
			ret = TRUE;
		if (Init == TRUE)
			Error_Read = 0;
	}
	UnLock_Wifly();
	return ret;
}
